% Note, u has to be positive in positive x dir, so assume vu is not 
% Also, calc must be done in system relative coords, so specify a
% background speed to subtract from (-vu)
disp('Running streamtrace...');
v=-vu-backspeed;

xr=xrmin:dxr:xrmax;
zr=zrmin:dzr:zrmax;
%allows tracing to get a streamline (or simple boundary) and calculates

%v=zap3(v,3,4,0);
v=mzap(v,3,4,0);

vr=minterp(v,range,elev,xr,zr);
vr(1,1)=NaN;
if numnan > 0
	vr=replace(vr,numnan);
end
if smoothing > 0
	vr=smooth2d(vr,smoothing/4,smoothing);
end

vr(2,:)=vr(1,:);
vr(length(xr),:)=vr(length(xr)-1,:);

shr=ddy(vr',zr)';


if trace==1

	figure(15)
	clf

	axis([xrmin-2 xrmax+2 zrmin zrmax]);
	hold on

	if underlay == 2
		p=pcolor(X,Z,v);
		caxis(fliplr(-[vmin vmax]-backspeed));
	elseif underlay == 3
		p=pcolor(xr,zr,shr');
		caxis([shmin shmax]);
		disp('set shear shading range with [shmin shmax] before doing this!')
	elseif underlay == 4
		p=pcolor(xr,zr,vr');
		caxis(fliplr(-[vmin vmax]-backspeed));
	else
		p=pcolor(X,Z,sh);
		caxis([shmin shmax]);
		disp('set shear shading range with [shmin shmax] before doing this!')
	end
	shading flat
	hold on;
	
	button=1;
	i=0;
	
	disp('Click along a streamline (from l to r)');
	disp('Make sure the first point is less than xmin, the last is greater than xmax')
	
	[x,z,button]=ginput2(1);

	while x >= xrmin,
		disp('x value is greater than xmin')
		[x,z,button]=ginput2(1);
	end

	ph=[];
	xin=[];
	zin=[];
	again=1;
	
	while again == 1
	
		i=i+1;
		
		xin(i)=x;
		zin(i)=z;
		
		ph(i,:)=plot(x,z,'kx',x,z,'wo')';
		
		if x > xrmax
			again=0;
		else
			[x,z,button]=ginput2(1);
		end
		
	end

	disp('Done inputting points')

	delete(ph);

%	shading flat

	zrout=interp1(xin,zin,xr,'cubic');
	dzdx=gradient(zrout,dxr);
	zi=1+round((zrout-zmin)/dzr);

%	ph2=plot(xr,zrout,'k');

elseif trace == 2

	zrout=xr*0+boundz;
	dzdx=0*xr;
	zi=1+round((zrout-zmin)/dzr);

end

%might as well treat streamline as zero, say

sf=vr*NaN;
[xrg zrg]=meshgrid(xr,zr);
dens=1.2*exp(-zrg/8.0)';

%Force boundary condition?

%integrate away from a level, not worrying about boundary conditions
if ground == 1
%go down....
	for i=1:length(xr)
		sf(i,zi(i))=0-vr(i,zi(i))*dens(i,zi(i))*(zrout(i)-zr(zi(i)));
		for j=zi(i):-1:2
			 sf(i,j-1)=sf(i,j)-dzr*vr(i,j)*dens(i,j);
		end
	end

%now up......
	for i=1:length(xr)
		for j=zi(i):(length(zr)-1)
			 sf(i,j+1)=sf(i,j)+dzr*vr(i,j)*dens(i,j);
		end
	end
	wr2=-ddx(sf',xr)'./dens;
	vr2=ddy(sf',zr)'./dens;
	wr=wr2;

%integrate away from a level, trying to get zero w at ground
elseif ground == 2

	if trace==2

		sf1=zeros(size(vr));
	
		nx=length(xr);
		nz=zi(1);
	
		i=1:nx;
			for j=1:nz-1;
			 sf1(i,j+1)=sf1(i,j)+0.5*dzr*(vr(i,j)+vr(i,j+1)).*dens(i,j);
		end
	
		w=-ddx(sf1',xr)'./dens;
		w(nx,:)=w(nx-1,:);
		w(1,:)=w(2,:);
		wn=w*0;
	
		wt=0;  %upper BC (allow this to be a vector?)
		zt=zr(nz);
		for j=1:nz
			wn(i,j)=w(i,j)-(zrg(j,i)'/zt).*(w(i,j)-wt);
		end

method=2  % 1 for nothing, 2 for poisson

if method==1

disp('not calculating sf properly, but wr is the adjusted vertical velocity')
sf=sf1;
vr2=vr;
wr=wn;

else
	disp('doing the fishy stuff')
		vort=dens.*(-ddx(wn',xr)'+ddy(vr',zr)'-0.125); %the vorticity function - includes density term for mass-weighted streamfunction...

		psi=vr*0;
		
%now work out boundary conditions:
%bottom is psi=0
%top is given by the mean mass flux...

		psi_top=sum(sum(dens(:,1:nz).*vr(:,1:nz)))/(nx*nz);
		psi_bot=0;

		if (1)
			psi=mfish(vort(:,1:nz), dxr, dzr, psi_top, psi_bot);
		else

			psi=vr*0;					
			psi(:,1)=psi_bot;
			psi(:,nz)=psi_top;
			
			%left and right are going to be given by vertical velocity=0..., and
			%we will update psi at the boundary
			
			i=2:nx-1;
			j=2:nz-1;
			i2=[1,nx];
			dx2=dxr^2;
			dz2=dzr^2;		
			dt=min([dx2/4 dz2/4]);
			dtdx2=dt/dx2;
			dtdz2=dt/dz2;
			
			for count=1:1000
%				psi(4,6)
				psi(i,j)=psi(i,j)+dtdx2*(psi(i+1,j)+psi(i-1,j)-2*psi(i,j))+dtdz2*(psi(i,j+1)+psi(i,j-1)-2*psi(i,j))-dt*vort(i,j);
				psi(1,j)=psi(1,j)+dtdx2*(psi(2,j)-psi(1,j))+dtdz2*(psi(1,j+1)+psi(1,j-1)-2*psi(1,j))-dt*vort(1,j);
				psi(nx,j)=psi(nx,j)+dtdx2*(-psi(nx,j)+psi(nx-1,j))+dtdz2*(psi(nx,j+1)+psi(nx,j-1)-2*psi(nx,j))-dt*vort(nx,j);
				
			end
		end % how to solve poisson
		sf(:,1:nz)=psi;
		wr=wn;
		wr2=-ddx(sf',xr)'./dens;
		vr2=ddy(sf',zr)'./dens;
		
end

	else %otherwise use the "wrong" method of fudging the streamlines

		sf=zeros(size(vr));
	
%go down...
		for i=1:length(xr)
			sf(i,zi(i))=0-vr(i,zi(i))*dens(i,zi(i))*(zrout(i)-zr(zi(i)));
			for j=zi(i):-1:2
				 sf(i,j-1)=sf(i,j)-dzr*vr(i,j)*dens(i,j);
			end
		end
	
%going to use this in a bit, don't want NaNs in it
		wr=-ddx(sf',xr)'./dens;
	
%now up......
		for i=1:length(xr)
			for j=zi(i):(length(zr)-1)
				 sf(i,j+1)=sf(i,j)+dzr*vr(i,j)*dens(i,j);
			end
		end

%now correct it, adding on a bit to the streamline...

		zimin=1;
	
		weights=1./mean(10+wr'.^2);
		weights=length(xr)*weights/sum(weights);
		meansf=round(mean(sf(:,zimin).*weights'));
	
		dsf=(sf(:,zimin)-meansf)'./(zi-zimin);
	
		sf2=sf;
	
		for i=1:length(xr)
			for j=zimin:zi(i)-1
				sf2(i,j)=sf(i,j)-dsf(i)*(zi(i)-j);
			end
		end
	
		if zmin > 1
			sf2(:,1:zimin-1)=NaN;
		end
	
		sf2(length(xr),:)=NaN;
	
		sf=sf2;

		wr2=-ddx(sf',xr)'./dens;
		vr2=ddy(sf',zr)'./dens;

	end

end

disp('done calculations... now plotting preview')

overlay=0;streamoverlay;